/** theme-sorting_desk-default.css --- Default theme used by examples
 * illustrating usage of the SortingDesk component.
 *
 * Copyright (C) 2014 Diffeo
 *
 * Comments:
 *
 *
 */

/*csslint adjoining-classes: false, important: false */


/* HTML tags */
BODY {
  font: 15px Arial, sans-serif;
}

BODY.sd-droppable-hover {
  height: 96vh;
  border: none !important;
  background: radial-gradient(ellipse at center, rgba(244,244,240,1)
                              0%,rgba(255,255,255,0) 100%);
}

#wrapper-outer {
  width: 1000px;
  margin: auto;
}

.wrapper {
  position: absolute;
  border: 4px solid #DFDED5;
  border-radius: 10px;
  display: none;
}

#wrapper-items {
  bottom: 10px;
  left: 50%;
  margin-left: -300px;
  width: 600px;
  background-color: white;
}

#wrapper-bins {
  width: 1000px;
  border: none;
}

#wrapper-items,
#wrapper-bins {
  overflow: auto;
}

#items, #bins {
  padding: 10px;
  overflow: auto;
  position: relative;
}

#loading {
  position: fixed;
  top: 20px;
  right: 20px;
  padding: 5px 10px 5px 10px;
  display: none;
  background-color: rgb(219, 60, 60);
  color: white;
}

#notification {
  display: none;
  width: 600px;
  position: absolute;
  left: 50%;
  margin-left: -300px;
}


/* SortingDesk specifics
 * ----------------------------------------------------------------------
 *  Bin */
.sd-bin {
  position: relative;
  border: 2px solid #7582C9;
  background-color: #D0DDFF;
  color: #5C7CCA;
  border-radius: 8px;
  padding: 15px;
  text-align: center;
  font-size: 16px;
  box-shadow: 1px 1px 1px 0 rgba(50, 50, 50, 0.75);
  text-shadow: 1px 1px 1px rgba(150, 150, 150, 1);
  transition: background-color 175ms, color 100ms;
}

.sd-bin:hover:not(.sd-adding) {
  background-color: #A9BEF3;
  color: #F2F6FF;
  cursor: pointer;
  transition: background-color 175ms, color 100ms;
}

.sd-bin:active:not(.sd-adding),
.sd-bin.sd-assign {
  position: relative;
  top: 3px;
  left: 3px;
  box-shadow: none;
  transition: background-color 175ms, color 100ms;
}

/* Bin: common */
.sd-bin {
  float: left;
  margin: 10px 20px 10px 0;
  text-align: center;

  box-shadow: 1px 1px 5px 0 rgba(50, 50, 50, 0.75);
/*   text-shadow: 1px 1px 1px rgba(150, 150, 150, 1); */
  transition: background-color 175ms, color 100ms;
}

.sd-bin {
  word-wrap: break-word;
}

.sd-bin.sd-active {
  color: #E6FFD6;
  background-color: #83c78e;
  border-color: #349950;
}

.sd-bin.sd-active:hover {
  color: #E6FFD6;
  background-color: #92E29F;
  border-color: #349950;
}

/* Bin: shortcut */
.sd-bin-shortcut {
  position: absolute;
  top: 0;
  right: 0;
  width: 15px;
  height: 15px;
  border-left: 1px solid silver;
  border-bottom: 1px solid silver;
  border-bottom-left-radius: 5px;
  font-size: 11px;
}

.sd-bin .sd-bin-shortcut {
  border-color: #7582C9;
}

/* Text item */
.sd-text-item {
  position: relative;
  margin: 15px 5px;
  padding: 10px;
  background-color: #FFFFFA;
  border: 2px solid #E0E2B6;
  border-radius: 8px;
  color: #242922;
  text-align: justify;
  font-size: 14px;
  cursor: default;
}

.sd-text-item B:not(:first-child),
.sd-text-item STRONG:not(:first-child) {
  padding-left: 10px;
}

.sd-text-item-content {
  overflow: hidden;
  max-height: 65px;
  text-overflow: ellipsis;
}

.sd-text-item-content * {
  display: inline-block;
}

.sd-text-item:hover {
  background-color: #F8F8DD;
  color: #44573C;
}

.sd-text-item:hover .sd-text-item-title {
  background-color: #F8F8DD;
}

.sd-text-item:active {
  background-color: #FFF5F8;
  color: #7F8368;
}

.sd-text-item:active .sd-text-item-title {
  background-color: #FFF5F8;
}

.sd-text-item.sd-selected {
  border: 2px solid #8EB4DA;
}

.sd-text-item:selected .sd-text-item-title {
  background-color: #FFF5F8;
}

.sd-text-item.sd-selected:hover {
  background-color: inherit;
}

.sd-more, .sd-less {
  float: right;
  font-size: 85%;
  padding-top: 3px;
  color: #999;
}

.sd-more:hover, .sd-less:hover {
  color: #123;
  cursor: pointer;
}

A.sd-text-item-title {
  position: absolute;
  top: -12px;
  background-color: white;
  padding: 2px 8px 2px 8px;
  display: block;
  text-decoration: none;
  color: #2D5C8B;
  border-radius: 6px;
}

A.sd-text-item-title:hover {
  color: #4A8BCC;
}

.sd-text-item:hover .sd-text-item-close,
.sd-text-item.sd-selected .sd-text-item-close {
  display: block;
}

.sd-text-item.sd-dragging:hover .sd-text-item-close,
.sd-text-item.sd-selected.sd-dragging .sd-text-item-close {
  display: none;
}

.sd-text-item-close {
  position: absolute;
  top: -8px;
  right: -6px;
  background-color: white;
  padding: 5px 4px 5px 5px;
  display: none;
  text-decoration: none;
  color: grey;
  border: 1px solid silver;
  border-radius: 12px;
  line-height: 7px;
  box-shadow: 1px 1px 1px 0 rgba(50, 50, 50, 0.75);
}

.sd-text-item-close:hover {
  color: black;
  background-color: #ddd;
}

.sd-text-item-close:active {
  background-color: #eee;
}

.sd-text-item.sd-dragging {
  border-color: #eee !important;
  color: silver !important;
  background-color: inherit !important;
}

/* Droppable elements */
.sd-droppable {
  border-style: dashed;
}

.sd-droppable-highlight {
  border-style: dashed !important;
}

.sd-droppable-hover {
  border-style: dotted !important;
}

/* Button: add */
.sd-button-add {
  position: relative;
  border: 2px solid #A2C5A6;
  background-color: #ECF8EA;
  clear: both;
  padding: 5px;
  margin: 15px auto 0 auto;
  font-size: 25px;
  color: green;
  font-weight: bold;
  width: 28px;
  text-align: center;
  border-radius: 19px;
  box-shadow: 1px 1px 2px 0 rgba(50, 50, 50, 0.75);
  text-shadow: 1px 1px 2px rgba(150, 150, 150, 1);
  transition: background-color 175ms, color 100ms;
  cursor: pointer;
  line-height: 25px;
}

.sd-button-add:hover {
/*  color: #67B285; */
  background-color: #DEF3DA;
  transition: background-color 175ms, color 100ms;
}

.sd-button-add:active {
  top: 1px;
  left: 1px;
  box-shadow: none;
  transition: background-color 175ms, color 100ms;
}

/* Button: dismiss */
.sd-button-dismiss {
  position: fixed;
  top: 50%;
  left: 50%;
  margin-top: -29px;
  margin-left: -29px;
  border: 2px solid #C5A2A2;
  background-color: #F8EAEA;
  clear: both;
  font-size: 20px;
  color: #900;
  padding: 15px 6px 15px 6px;
  width: 40px;
  text-align: center;
  border-radius: 30px;
  box-shadow: 1px 1px 2px 0 rgba(50, 50, 50, 0.75);
  text-shadow: 1px 1px 2px rgba(150, 150, 150, 1);
  transition: background-color 175ms, color 100ms;
  cursor: pointer;
  display: none;
}

.sd-button-dismiss:hover {
  background-color: #F3DADA;
  transition: background-color 175ms, color 100ms;
}

.sd-button-dismiss.droppable-hover {
  background-color: #C04343;
  border-color: #7A4747;
  color: #FFF5F5;
}
